-- Import necessary modules and libraries
require "import"
import "android.app.*"
import "android.os.*"
import "android.widget.*"
import "android.view.*"
import "android.net.*"
import "android.text.*"
import "android.media.MediaPlayer"
import "java.io.BufferedReader"
import "java.io.File"
import "java.io.FileReader"

-- Create an activity
local activity = activity
activity.title = "IPTV Player"

-- Define the path to your M3U file (modify this to your desired file path)
local filePath = "storage/emulated/0/解说/Tools/iptv player/iptv.txt"

local mainLayout = LinearLayout(activity)
mainLayout.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT)
mainLayout.orientation = LinearLayout.VERTICAL

-- Create a VideoView to display the IPTV stream
local videoView = VideoView(activity)
videoView.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT)

-- Create a table to store IPTV URLs and names
local iptvUrl = {}
-- Function to extract IPTV URLs from an M3U file and store them in the table
function extractIPTVUrls(filePath)
    -- Check if the file exists
    local file = File(filePath)
    if file.exists() then
        local reader = BufferedReader(FileReader(file))
        local line1 = reader.readLine()
        local line2
while line1 do
            line2 = reader.readLine() -- Read the next line for the URL
            if line2 and string.match(line2, "^https?://") then
                -- Extract the value after "tvg-id=" from the first line and set it as the channel name
                local channelName = string.match(line1, 'tvg%-id="(.-)"')
                if channelName then
                    table.insert(iptvUrl, {name = channelName, url = line2}) -- Store the URL and name in a table
                end
            end
            line1 = reader.readLine() -- Read the next line for the next URL name
        end
reader.close()
    else
        -- Display a toast message if the file doesn't exist
        Toast.makeText(activity, "M3U file does not exist.", Toast.LENGTH_SHORT).show()
    end
end
-- Extract IPTV URLs from the M3U file and store them in the table
extractIPTVUrls(filePath)

-- Define a variable to keep track of the current IPTV URL index
local currentUrlIndex = 1
-- Create an AutoCompleteTextView for search input channel name
local autoCompleteTextView = AutoCompleteTextView(activity)
autoCompleteTextView.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
autoCompleteTextView.hint = "search channel"
-- Create a ListView to display search results
local listView = ListView(activity)
listView.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT)
-- Create a TextChangedListener for AutoCompleteTextView
local CustomTextChangedListener = TextWatcher {
    onTextChanged = function(s, start, before, count)
        searchChannelsByName(s.toString())
    end
}
-- Add a TextChangedListener to the AutoCompleteTextView to dynamically search channels
autoCompleteTextView.addTextChangedListener(CustomTextChangedListener)

-- Function to search and filter channels by name
function searchChannelsByName(searchTerm)
    local results = {}
    for _, channel in ipairs(iptvUrl) do
        if string.find(channel.name:lower(), searchTerm:lower()) then
            table.insert(results, channel.name)
        end
    end
    updateListViewWithResults(results)
end

-- Function to play the channel based on its name
function playChannelByName(channelName)
    for _, channel in ipairs(iptvUrl) do
        if channel.name == channelName then
            local channelIndex = getChannelIndexByName(channelName) -- Correct the typo in "selevtedChannelName"
            if channelIndex then
                playChannelAtIndex(channelIndex)
                break
            end
        end
    end
end

-- Function to get the index of a channel by its name
function getChannelIndexByName(selectedChannelName)
    for i, channel in ipairs(iptvUrl) do
        if channel.name == selectedChannelName then
            return i
        end
    end
    return nil
end

-- Function to play a channel at a specific index in iptvUrls
function playChannelAtIndex(index)
    if index <= #iptvUrl then
        videoView.setVideoURI(Uri.parse(iptvUrl[index].url))
        videoView.start()
        channelNameTextView.text = iptvUrl[index].name
        showToastWithChannelName(iptvUrl[index].name)
        activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
    end
end

-- Function to update the ListView with search results
function updateListViewWithResults(results)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, results)
    listView.setAdapter(adapter)
    listView.setOnItemClickListener(AdapterView.OnItemClickListener {
        onItemClick = function(parent, view, position, id)
            local selectedChannelName = results[position + 1] -- Adjust for 0-based indexing
            playChannelByName(selectedChannelName)
        end
    })
end

-- Function to display a Toast message with the channel name
function showToastWithChannelName(channelName)
    local toast = Toast.makeText(activity, "Now playing: " .. channelName, Toast.LENGTH_SHORT)
    toast.show()
end

-- Function to play the channel based on its name and return the URL
function playChannelByName(channelName)
    for _, channel in ipairs(iptvUrl) do
        if channel.name == channelName then
            local channelIndex = getChannelIndexByName(channelName)
            if channelIndex then
                local channelUrl = iptvUrl[channelIndex].url
                playChannelByURL(channelUrl)
                local toastMessage = "Playing: " .. channelName
                showChannelNameToast(toastMessage) -- Show a toast for the channel name
                return channelUrl -- Return the URL of the selected channel
            end
        end
    end
end

-- Function to play a channel by URL
function playChannelByURL(channelUrl)
    videoView.setVideoURI(Uri.parse(channelUrl))
    videoView.start()
    -- You can also update other UI elements or perform any additional actions here.
    activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
end

-- Function to show a toast message
function showChannelNameToast(message)
    local Toast = luajava.bindClass("android.widget.Toast")
    local toast = Toast.makeText(activity, message, Toast.LENGTH_SHORT)
    toast.show()
end

-- Usage example to play a selected channel and get its URL
local selectedChannelName = "Channel Name" -- Replace with the actual selected channel name
local selectedChannelURL = playChannelByName(selectedChannelName)
if selectedChannelURL then
    -- You have the URL of the selected channel, and it has been played.
    -- You can use the selectedChannelURL as needed.
end

-- Create a Spinner to select categories
local categorySpinner = Spinner(activity)
local categoryAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
categoryAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
categorySpinner.setAdapter(categoryAdapter)
mainLayout.addView(categorySpinner)

-- Create a table to store categories and their corresponding channels
local categories = {}

-- Function to organize channels by categories
function organizeChannelsByCategory(filePath)
    local file = File(filePath)
    if file.exists() then
        local reader = BufferedReader(FileReader(file))
        local line1
        local currentCategory = nil

        while true do
            line1 = reader.readLine() -- Read the first line
            if not line1 then
                break
            end

            local category = string.match(line1, 'title="(.-)"')
            if category then
                currentCategory = category
                if not categories[currentCategory] then
                    categories[currentCategory] = {} -- Initialize the category with an empty table
                    categoryAdapter.add(currentCategory) -- Add category to the Spinner
                end
            end

            local line2 = reader.readLine()
            if line2 and string.match(line2, "^http?://" and "^https?://") then
                local channelName = string.match(line1, 'tvg%-id="(.-)"')
if channelName  then
                    table.insert(categories[currentCategory], {url = line2, 
name = channelName})
                end
            end
        end

        reader.close()
    else
        -- Display a toast message if the file doesn't exist
        Toast.makeText(activity, "M3U file does not exist.", Toast.LENGTH_SHORT).show()
    end
end

-- Call the function to organize channels by category
organizeChannelsByCategory(filePath)

-- Set a selection listener for the Spinner to filter channels
categorySpinner.setOnItemSelectedListener(AdapterView.OnItemSelectedListener {
    onItemSelected = function(parent, view, position, id)
        local selectedCategory = categoryAdapter.getItem(position)
        updateIPTVUrlsByCategory(selectedCategory)
        currentUrlIndex = 1  -- Reset the current index to the first channel of the selected category
        playChannelAtIndex(currentUrlIndex)
    end,
    onNothingSelected = function(parent)
        -- Handle case when nothing is selected, if needed
    end
})

-- Create a table to store IPTV URLs and names
local iptvUrls = {}

-- Add buttons to your activity layout
local buttonLayout = LinearLayout(activity)
buttonLayout.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
buttonLayout.orientation = LinearLayout.HORIZONTAL
-- Create previous, play/pause, and next buttons
local previousButton = Button(activity)
previousButton.text = "Previous"
local playPauseButton = Button(activity)
playPauseButton.text = "Play/Pause"
local nextButton = Button(activity)
nextButton.text = "Next"
-- Add buttons to the layout
local channelNameTextView = TextView(activity)
channelNameTextView.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
channelNameTextView.gravity = Gravity.CENTER
--channelNameTextView.text = iptvUrls[currentUrlIndex].name   -- Set the initial text to the first channel name
-- Add the TextView to the main layout
-- Add it as the first child in the layout

-- Add a click listener for the previous button
previousButton.onClick = function()
    playPreviousChannel()
end

-- Add a click listener for the play/pause button
playPauseButton.onClick = function()
    if videoView.isPlaying() then
        videoView.pause()
    else
        videoView.start()
        showToastWithChannelName(iptvUrls[currentUrlIndex].name)
    end
end

-- Add a click listener for the next button
nextButton.onClick = function()
    playNextChannel()
end

-- Function to update the iptvUrls table based on the selected category
function updateIPTVUrlsByCategory(selectedCategory)
    iptvUrls = categories[selectedCategory] or ""
end

-- Function to display a Toast message with the channel name
local function showToastWithChannelName(channelName)
    local toast = Toast.makeText(activity, "Now playing: " .. channelName, Toast.LENGTH_SHORT)
    toast.show()
end
local currentUrlIndex = 1

-- Function to play a channel at a specific index in iptvUrls
function playChannelAtIndex(index)
    if index <= #iptvUrls then
        videoView.setVideoURI(Uri.parse(iptvUrls[index].url ))
        videoView.start()
        channelNameTextView.text = iptvUrls[index].name 
        showToastWithChannelName(iptvUrls[index].name)
         activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        
end
end

-- Function to play the next channel
function playNextChannel()
    if currentUrlIndex < #iptvUrls then
        currentUrlIndex = currentUrlIndex + 1
        playChannelAtIndex(currentUrlIndex)
else
        Toast.makeText(activity, "No next channel.", Toast.LENGTH_SHORT).show()
    end
end

-- Function to play the previous channel
function playPreviousChannel()
    if currentUrlIndex > 1 then
        currentUrlIndex = currentUrlIndex - 1
        playChannelAtIndex(currentUrlIndex)
else
        Toast.makeText(activity, "No previous channel.", Toast.LENGTH_SHORT).show()
    end
end

-- Add touch event handling to detect swipe gestures
local initialX = 0
local initialY = 0
mainLayout.setOnTouchListener(View.OnTouchListener {
    onTouch = function(view, event)
        if event.getAction() == MotionEvent.ACTION_DOWN then
            initialX = event.getX()
            initialY = event.getY()
            return true
        elseif event.getAction() == MotionEvent.ACTION_UP then
            local finalX = event.getX()
            local finalY = event.getY()
            local deltaX = finalX - initialX
            local deltaY = finalY - initialY
if math.abs(deltaY) > 50 then
                if deltaY < 0 then
                    playNextChannel()
                else
                    playPreviousChannel()
                end
            end
            return true
        end
        return false
    end
})

mainLayout.addView(videoView)

mainLayout.addView(buttonLayout)

--buttonLayout.addView(previousButton)
buttonLayout.addView(playPauseButton)
--buttonLayout.addView(nextButton)
 

--buttonLayout.addView(channelNameTextView, 0)

mainLayout.addView(autoCompleteTextView)

buttonLayout.addView(listView)

activity.setContentView(mainLayout)